@@ -12,6 +12,9 @@ module Network.HTTP.Client.Core
     , httpRedirect
     ) where
 
+#if !MIN_VERSION_base(4,6,0)
+import Prelude hiding (catch)
+#endif
 import Network.HTTP.Types
 import Network.HTTP.Client.Manager
 import Network.HTTP.Client.Types
@@ -146,13 +149,16 @@ httpRaw req0 m = do
 --
 -- Since 0.1.0
 responseOpen :: Request -> Manager -> IO (Response BodyReader)
-responseOpen req0 manager = mWrapIOException manager $ do
+responseOpen req0 manager = handle addTlsHostPort $ mWrapIOException manager $ do
     res <-
         if redirectCount req0 == 0
             then httpRaw req0 manager
             else go (redirectCount req0) req0
     maybe (return res) throwIO =<< applyCheckStatus req0 (checkStatus req0) res
   where
+    addTlsHostPort (TlsException e) = throwIO $ TlsExceptionHostPort e (host req0) (port req0)
+    addTlsHostPort e = throwIO e
+
     go count req' = httpRedirect
       count
       (\req -> do
@@ -187,6 +193,9 @@ applyCheckStatus req checkStatus' res =
             responseClose res
             return (Just exc')
   where
+#ifndef MIN_VERSION_bytestring
+#define MIN_VERSION_bytestring(x,y,z) 1
+#endif
 #if MIN_VERSION_bytestring(0,10,0)
     toStrict' = L.toStrict
 #else
@@ -212,6 +221,10 @@ httpRedirect count0 http' req0 = go count0 req0 []
                 -- instead just close the connection.
                 let maxFlush = 1024
                 lbs <- brReadSome (responseBody res) maxFlush
+                    -- The connection may already be closed, e.g.
+                    -- when using withResponseHistory. See
+                    -- https://github.com/snoyberg/http-client/issues/169
+                    `catch` \(_ :: ConnectionClosed) -> return L.empty
                 responseClose res
 
                 -- And now perform the actual redirect
