@@ -8,7 +8,7 @@ module Network.HTTP.Client.Response
     , lbsResponse
     ) where
 
-import Control.Monad ((>=>))
+import Control.Monad ((>=>), when)
 
 import Control.Exception (throwIO)
 
@@ -17,8 +17,10 @@ import qualified Data.ByteString.Lazy as L
 
 import Data.Default.Class (def)
 
+import Data.Maybe (isJust)
+
 import qualified Network.HTTP.Types as W
-import Network.URI (parseURIReference)
+import Network.URI (parseURIReference, escapeURIString, isAllowedInURI)
 
 import Network.HTTP.Client.Types
 
@@ -27,8 +29,6 @@ import Network.HTTP.Client.Util
 import Network.HTTP.Client.Body
 import Network.HTTP.Client.Headers
 
-import System.Timeout (timeout)
-
 -- | If a request is a redirection (status code 3xx) this function will create
 -- a new request from the old request, the server headers returned with the
 -- redirection, and the redirection code itself. This function returns 'Nothing'
@@ -56,7 +56,8 @@ getRedirectedRequest :: Request -> W.ResponseHeaders -> CookieJar -> Int -> Mayb
 getRedirectedRequest req hs cookie_jar code
     | 300 <= code && code < 400 = do
         l' <- lookup "location" hs
-        req' <- setUriRelative req =<< parseURIReference (S8.unpack l')
+        let l = escapeURIString isAllowedInURI (S8.unpack l')
+        req' <- setUriRelative req =<< parseURIReference l
         return $
             if code == 302 || code == 303
                 -- According to the spec, this should *only* be for status code
@@ -86,14 +87,12 @@ getResponse :: ConnRelease
             -> Maybe Int
             -> Request
             -> Connection
+            -> Maybe (IO ()) -- ^ Action to run in case of a '100 Continue'.
             -> IO (Response BodyReader)
-getResponse connRelease timeout'' req@(Request {..}) conn = do
-    let timeout' =
-            case timeout'' of
-                Nothing -> id
-                Just t -> timeout t >=> maybe (throwIO ResponseTimeout) return
-    StatusHeaders s version hs <- timeout' $ parseStatusHeaders conn
+getResponse connRelease timeout' req@(Request {..}) conn cont = do
+    StatusHeaders s version hs <- parseStatusHeaders conn timeout' cont
     let mcl = lookup "content-length" hs >>= readDec . S8.unpack
+        isChunked = ("transfer-encoding", "chunked") `elem` hs
 
         -- should we put this connection back into the connection manager?
         toPut = Just "close" /= lookup "connection" hs && version > W.HttpVersion 1 0
@@ -101,12 +100,11 @@ getResponse connRelease timeout'' req@(Request {..}) conn = do
 
     body <-
         -- RFC 2616 section 4.4_1 defines responses that must not include a body
-        if hasNoBody method (W.statusCode s) || mcl == Just 0
+        if hasNoBody method (W.statusCode s) || (mcl == Just 0 && not isChunked)
             then do
                 cleanup True
                 return brEmpty
             else do
-                let isChunked = ("transfer-encoding", "chunked") `elem` hs
                 body1 <-
                     if isChunked
                         then makeChunkedReader rawBody conn
