@@ -7,6 +7,7 @@ module Network.HTTP.Client.Types
     ( BodyReader
     , Connection (..)
     , StatusHeaders (..)
+    , ConnectionClosed (..)
     , HttpException (..)
     , Cookie (..)
     , CookieJar (..)
@@ -22,6 +23,7 @@ module Network.HTTP.Client.Types
     , Response (..)
     , ResponseClose (..)
     , Manager (..)
+    , HasHttpManager (..)
     , ConnsMap (..)
     , ManagerSettings (..)
     , NonEmptyList (..)
@@ -54,6 +56,7 @@ import qualified Data.Map as Map
 import Data.Text (Text)
 import Data.Streaming.Zlib (ZlibException)
 import Control.Concurrent.MVar (MVar)
+import Data.CaseInsensitive as CI
 
 -- | An @IO@ action that represents an incoming response body coming from the
 -- server. Data provided by this action has already been gunzipped and
@@ -73,12 +76,20 @@ data Connection = Connection
     , connectionWrite :: S.ByteString -> IO ()
       -- ^ Send data to server
     , connectionClose :: IO ()
+      -- ^ Close connection. Any successive operation on the connection
+      -- (exept closing) should fail with `ConnectionClosed` exception.
+      -- It is allowed to close connection multiple times.
     }
     deriving T.Typeable
 
 data StatusHeaders = StatusHeaders Status HttpVersion RequestHeaders
     deriving (Show, Eq, Ord, T.Typeable)
 
+data ConnectionClosed = ConnectionClosed
+  deriving (Eq, Show, T.Typeable)
+
+instance Exception ConnectionClosed
+
 data HttpException = StatusCodeException Status ResponseHeaders CookieJar
                    | InvalidUrlException String String
                    | TooManyRedirects [Response L.ByteString]  -- ^ List of encountered responses containing redirects in reverse chronological order; including last redirect, which triggered the exception and was not followed.
@@ -99,7 +110,7 @@ data HttpException = StatusCodeException Status ResponseHeaders CookieJar
                    -- on this, see:
                    --
                    -- <https://github.com/snoyberg/http-client/commit/b86b1cdd91e56ee33150433dedb32954d2082621#commitcomment-10718689>
-                   | FailedConnectionException2 String Int Bool SomeException -- ^ host/port/secure
+                   | FailedConnectionException2 String Int Bool SomeException -- ^ host\/port\/secure
                    | ExpectedBlankAfter100Continue
                    | InvalidStatusLine S.ByteString
                    | InvalidHeader S.ByteString
@@ -131,6 +142,10 @@ data HttpException = StatusCodeException Status ResponseHeaders CookieJar
                    -- and @transfer-encoding: chunked@ are used. Since 0.4.8.
                    --
                    -- Since 0.4.11 this exception isn't thrown anymore.
+                   | TlsExceptionHostPort SomeException S.ByteString Int
+                   -- ^ TLS exception, together with the host and port
+                   --
+                   -- @since 0.4.24
     deriving (Show, T.Typeable)
 instance Exception HttpException
 
@@ -158,7 +173,7 @@ newtype CookieJar = CJ { expose :: [Cookie] }
 instance Eq Cookie where
   (==) a b = name_matches && domain_matches && path_matches
     where name_matches = cookie_name a == cookie_name b
-          domain_matches = cookie_domain a == cookie_domain b
+          domain_matches = CI.foldCase (cookie_domain a) == CI.foldCase (cookie_domain b)
           path_matches = cookie_path a == cookie_path b
 
 instance Ord Cookie where
@@ -595,6 +610,11 @@ data Manager = Manager
     }
     deriving T.Typeable
 
+class HasHttpManager a where
+    getHttpManager :: a -> Manager
+instance HasHttpManager Manager where
+    getHttpManager = id
+
 data ConnsMap
     = ManagerClosed
     | ManagerOpen {-# UNPACK #-} !Int !(Map.Map ConnKey (NonEmptyList Connection))
